#!/usr/bin/env python
# -*- coding: latin-1 -*-
#
# @module towers.core.validation
# @version 0.1
# @copyright (c) 2017-present Francis Horsman.


import abc

import six

from .errors import InvalidMoves, InvalidRods, InvalidTowerHeight

__all__ = [
    'Validatable',
    'validate_height',
    'validate_rods',
    'validate_moves',
]


class Validatable(object):
    @abc.abstractmethod
    def validate(self):
        """
        Perform self validation
        """
        raise NotImplementedError()


def validate_height(height):
    """
    Validate the height of a :class:`Tower`s or :class:`Rod`.

    :param int height:
        The height to validate.
    :raises InvalidTowerHeight:
        The height of the :class:`Tower` is invalid.
    """
    if height < 1:
        raise InvalidTowerHeight(height)


def validate_rods(rods):
    """
    Validate the rods.

    :param List[Rod]|None rods:
        The :class:`Rod`'s to validate.
    :raises InvalidRods:
        expecting type :class:`Rods`.
    :raises DuplicateDisk:
        This :class:`Rod` already contains this :class:`Disk`
    :raises CorruptRod:
        A :class:`Disk` is on top of a :class:`Disk` of smaller size.
    """
    from towers import Rods

    if rods is not None:
        if not isinstance(rods, Rods):
            raise InvalidRods(rods)
        rods.validate()


def validate_moves(moves):
    """
    Validate the number of moves.

    :param int moves:
        The moves count to validate.
    :raises InvalidMoves:
        The number of moves is not an number or is less than zero.
    """
    if moves is not None:
        if not isinstance(moves, six.integer_types):
            raise InvalidMoves(moves)
        if moves < 0:
            raise InvalidMoves(moves)



#!/usr/bin/env python
# -*- coding: latin-1 -*-
# pragma: no-cover

__title__ = 'towers'
__description__ = 'Towers of Hanoi for Python'
__url__ = 'http://towers.readthedocs.io/en/latest'
__version__ = '0.1.1'
__author__ = 'Francis Horsman'
__author_email__ = 'francis.horsman@gmail.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2017 Francis Horsman'



#!/usr/bin/env python
# -*- coding: latin-1 -*-
#
# @module towers.core.utils
# @version 0.1
# @copyright (c) 2017-present Francis Horsman.

import abc


class Serializable(object):
    """
    A mixin which shows that a class is serializable.
    """

    @abc.abstractmethod
    def to_json(self):
        """
        Return a json serializable representation of this instance.

        :rtype: object
        """
        raise NotImplementedError()

    @abc.abstractmethod
    def from_json(self, d):
        """
        Return a class instance from a json serializable representation.

        :param str|dict d:
            The json or decoded-json from which to create a new instance from.
        """
        raise NotImplementedError()



#!/usr/bin/env python
# -*- coding: latin-1 -*-
#
# @module towers.core.rod
# @version 0.1
# @copyright (c) 2017-present Francis Horsman.

import json
from collections import Iterable, Sized, namedtuple

import six

from .disk import Disk
from .errors import CorruptRod, DuplicateDisk
from .utils import Serializable
from .validation import Validatable

__all__ = ['Rod']


class Rod(
    namedtuple('Rod', ('name', 'disks', 'height')), Iterable, Sized, Validatable, Serializable
):
    """
    A single tower containing disks.
    """

    def __new__(cls, name, disks=None, height=0):
        """
        :param str name:
            The name of the rod.
        :param List[Disk] disks:
            (optional) mutatable list of `Disks`.
        :param int height:
            The height of the rod.
        :rtype: Rod
        :raises: See `Rod.validate`.
        """
        self = super(Rod, cls).__new__(cls, name, disks or [], height)
        self.validate()
        return self

    def to_json(self):
        """
        Return a json serializable representation of this instance.

        :rtype: object
        """
        return {
            'name': self.name,
            'height': self.height,
            'disks': [i.to_json() for i in self.disks],
        }

    @classmethod
    def from_json(cls, d):
        """
        Return a class instance from a json serializable representation.

        :param Union[str,dict] d:
            The json or decoded-json from which to create a new instance.
        :rtype: Rod
        :raises: See `Rod.__new__`.
        """
        if isinstance(d, six.string_types):
            d = json.loads(d)
        return cls(
            name=d.pop('name'),
            height=d.pop('height'),
            disks=[Disk.from_json(i) for i in d.pop('disks')],
        )

    def __len__(self):
        return self.height

    def __copy__(self):
        """
        Return a shallow copy of this instance.

        :rtype: Rod
        """
        return Rod(
            self.name,
            disks=self.disks,
            height=self.height,
        )

    def __deepcopy__(self, *d):
        """
        Return a deep copy of this instance.

        :param dict d:
            Memoisation dict.
        :rtype: Rod
        """
        return Rod(
            self.name,
            disks=self.disks[:],
            height=self.height,
        )

    def __str__(self):
        return '{name}({rod})'.format(
            name=self.name,
            rod=self.disks,
        )

    def __eq__(self, other):
        """
        Compare Rod instances for equivalence.

        :param Rod other:
        :rtype: bool
        """
        if isinstance(other, Rod):
            if other.height == self.height:
                if other.disks == self.disks:
                    return True

    def __bool__(self):
        """
        A Rod is considered True if it contains any disks.

        :rtype: bool
        """
        return self.__nonzero__()

    def __nonzero__(self):
        """
        A Rod is considered non-zero if it contains any disks.

        :rtype: bool
        """
        return bool(self.disks)

    def pop(self):
        """
        Pop the top most disk from this rod and return it

        :rtype: Disk
        """
        return self.disks.pop()

    def append(self, disk, validate=True):
        """
        Append the disk to this rod and optionally validate.

        :param Disk disk:
            The disk to add to the top of our rod.
        :param bool validate:
            True=perform self validation.
        """
        self.disks.append(disk)
        if validate:
            self.validate()

    def __iter__(self):
        """
        Iterate over all the disks in this rod.

        :rtype: Disk
        """
        for disk in self.disks:
            yield disk

    def validate(self):
        """
        Perform self validation.

        :raises DuplicateDisk:
            This rod already contains this disk
        :raises CorruptRod:
            A disk is on top of a disk of smaller size.
        :raises InvalidTowerHeight:
            The height of the tower is invalid.
        :raises InvalidDiskPosition:
            The position of the disk is invalid.
        """
        width = 0
        seen = set()

        for disk in reversed(list(iter(self))):
            disk_width = disk.width
            if disk_width in seen:
                raise DuplicateDisk(self, disk_width)
            seen.update([disk_width])

            if disk.width < width:
                raise CorruptRod(self, disk)
            width = disk_width

            disk.validate()



#!/usr/bin/env python
# -*- coding: latin-1 -*-
#
# @module towers.core.towers
# @version 0.1
# @copyright (c) 2017-present Francis Horsman.

import contextlib
import copy
import json
import math
from collections import Sequence

import six

from .errors import (
    InvalidEndingConditions, InvalidStartingConditions,
)
from .moves import Move
from .rod import Rod
from .rods import Rods
from .utils import Serializable
from .validation import (
    Validatable, validate_height, validate_moves, validate_rods,
)

__all__ = [
    'Towers',
]


class Towers(Sequence, Validatable, Serializable):
    """
    A representation of the towers including all logic.
    """

    class JsonEncoder(json.JSONEncoder):
        def default(self, obj):  # pylint: disable=E0202
            if isinstance(obj, Towers):
                return obj.to_json()
            else:  # pragma: no cover
                return json.JSONEncoder.default(self, obj)

    def __init__(self, height=1, rods=None, moves=0, verbose=False):
        """
        :param int height:
            The height of the towers (ie: max number of disks each one rod can hold).
        :param Rods rods:
            An existing :class:`Rods` instance to use with this :class:`Towers` (the heights must
            match).
        :param int moves:
            The number of moves already taken.
        :param verbose:
            True=enable verbose logging mode.
        """
        validate_height(height)
        validate_rods(rods)
        validate_moves(moves)
        self._rods = rods if rods is not None else Rods(height)
        self._moves = moves
        self._verbose = bool(verbose)

    def to_json(self):
        """
        Return a json serializable representation of this instance.

        :rtype: object
        """
        return {
            'height': self.height,
            'verbose': self.verbose,
            'moves': self.moves,
            'rods': self._rods.to_json(),
        }

    @classmethod
    def from_json(cls, d):
        """
        Return a class instance from a json serializable representation.

        :param str|dict d:
            The json or decoded-json from which to create a new instance.
        :rtype:
            Towers
        :raises:
            See :class:`Towers`.__new__.
        """
        if isinstance(d, six.string_types):
            d = json.loads(d)
        return cls(
            height=d.pop('height'),
            verbose=d.pop('verbose'),
            moves=d.pop('moves'),
            rods=Rods.from_json(d.pop('rods')),
        )

    @contextlib.contextmanager
    def context(self, reset_on_success=True, reset_on_error=False):
        """
        Create a temp context for performing moves.
        The state of this instance will be reset at context exit.

        :param bool reset_on_success:
            Reset this instance's state on exit from the context if no error occurred.
            Default = True.
        :param bool reset_on_error:
            Reset this instance's state on exit from the context if an error occurred.
            Default = False.
        """
        self.validate_start()

        verbose = self.verbose
        moves = self.moves
        rods = self._rods.to_json()

        try:
            yield self
            self.validate_end()
        except Exception:
            # Error inside context or validation:
            if reset_on_error:
                self._verbose = verbose
                self._moves = moves
                self._rods = Rods.from_json(rods)
        else:
            if reset_on_success:
                self._verbose = verbose
                self._moves = moves
                self._rods = Rods.from_json(rods)

    def __bool__(self):
        """
        A Towers is considered True if it's state is completed.

        :rtype:
            bool
        """
        return self.__nonzero__()

    def __nonzero__(self):
        """
        A Towers is considered non-zero if it's state is completed.

        :rtype:
            bool
        """
        try:
            self.validate_end()
            return True
        except Exception:
            return False

    def __copy__(self):
        """
        Return a shallow copy of this instance.

        :rtype:
            :class:`Towers`
        """
        return Towers(
            height=self.height,
            rods=self._rods,
            moves=self.moves,
            verbose=self.verbose,
        )

    def __deepcopy__(self, *d):
        """
        Return a deep copy of this instance.

        :param dict d:
            Memoisation dict.
        :rtype:
            :class:`Towers`
        """
        return Towers.from_json(self.to_json())

    def __eq__(self, other):
        """
        Compare Towers instances for equivalence.

        :param Towers other:
            The other :class:`Towers` to compare.
        :rtype:
            bool
        """
        if isinstance(other, Towers):
            if other.height == self.height:
                if other._rods == self._rods:
                    return True

    def __getitem__(self, index):
        """
        Get the :class:`Rod` at the given index.

        :param int index:
            The index to get the :class:`Rod` at.
        :rtype:
            Rod
        """
        return self._rods[index]

    def __contains__(self, x):
        """
        Does this :class:`Towers` contain the given :class:`Rod`.

        :param Rod x:
            The :class:`Rod` to find.
        :rtype:
            bool
        """
        if isinstance(x, Rod):
            return x in self._rods

    def __len__(self):
        """
        Determine how many :class:`Rod`'s this :class:`Towers` contains.

        :rtype:
            int
        """
        return len(self._rods)

    def __iter__(self):
        """
        Run the towers, yielding :class:`Move` instances.
        """
        for i in self.move_tower(
            height=self.height,
            start=self.start_rod,
            end=self.end_rod,
            tmp=self.tmp_rod,
        ):
            yield i

    def __str__(self):
        return 'Towers({rods})'.format(rods=self._rods)

    @property
    def verbose(self):
        """
        Obtain this instance's verbose flag.

        :rtype:
            bool
        """
        return self._verbose

    @verbose.setter
    def verbose(self, verbose):
        """
        Set this instance's verbose flag

        :param object verbose:
            True=enable verbose logging mode.
        """
        self._verbose = bool(verbose)

    @property
    def moves(self):
        """
        Determine how many moves have occurred so far.

        :rtype:
            int
        """
        return self._moves

    @property
    def height(self):
        """
        Obtain the height of the :class:`Towers` (ie: max number of disks each one rod can hold).

        :rtype:
            int
        """
        return self._rods.height

    @staticmethod
    def moves_for_height(height):
        """
        Determine the max number of moves required to solve the puzzle for the given height

        :param int height:
            The height of the :class:`Rods` (number of :class:`Disk` on a :class:`Rod`).
        :rtype: int
        """
        return int(math.pow(2, height)) - 1

    def validate_start(self):
        """
        Validate the start conditions for this towers

        :raises InvalidTowerHeight:
            The height of the :class:`Towers` is invalid
        :raises DuplicateDisk:
            This :class:`Rod` already contains this :class:`Disk`.
        :raises CorruptRod:
            A :class:`Disk` is on top of a :class:`Disk` of smaller size.
        :raises InvalidStartingConditions:
            Initial conditions are invalid.
        """
        validate_height(self.height)
        self._rods.validate()

        if not (bool(self._rods.start) and not bool(self._rods.end)):
            raise InvalidStartingConditions(self._rods, self.moves)

        if self.moves != 0:
            raise InvalidStartingConditions(self._rods, self.moves)

    def validate_end(self):
        """
        Validate the end conditions for this towers.

        :raises InvalidTowerHeight:
            The height of the tower is invalid
        :raises DuplicateDisk:
            This :class:`Rod` already contains this :class:`Disk`.
        :raises CorruptRod:
            A :class:`Disk` is on top of a :class:`Disk` of smaller size.
        :raises InvalidEndingConditions:
            End conditions are invalid.
        """
        validate_height(self.height)
        self._rods.validate()

        if not (bool(self._rods.end) and not bool(self._rods.start)):
            raise InvalidEndingConditions(self._rods)

    def validate(self):
        """
        Perform self validation.

        :raises InvalidTowerHeight:
            The height of the tower is invalid
        :raises DuplicateDisk:
            This :class:`Rod` already contains this :class:`Disk`.
        :raises CorruptRod:
            A :class:`Disk` is on top of a :class:`Disk` of smaller size.
        """
        validate_height(self.height)
        self._rods.validate()

    def __enter__(self):
        """
        Context-Manager entry, validate our entry state for towers-start conditions.

        :raises:
            See :func:`Towers.validate_start`.
        """
        self.validate_start()

    def __exit__(self, *args, **kwargs):
        """
        Context-Manager exit, validate our exit state for towers-end conditions.

        :raises:
            See :func:`Towers.validate_end`.
        """
        self.validate_end()

    def __call__(self):
        """
        Run the towers. Convenience method.

        :raises:
            See :func:`Towers.move_tower`.
        """
        for i in self:
            if self.verbose:
                print(i)

    @property
    def start_rod(self):
        """
        Retrieve the start :class:`Rod` for this towers.

        :rtype:
            Rod
        """
        return self._rods.start

    @property
    def end_rod(self):
        """
        Retrieve the end :class:`Rod` for this towers.

        :rtype:
            Rod
        """
        return self._rods.end

    @property
    def tmp_rod(self):
        """
        Retrieve the temporary :class:`Rod` for this towers.

        :rtype:
            Rod
        """
        return self._rods.tmp

    def move_tower(self, height, start, end, tmp):
        """
        Move the stack of `Disks` on a `Rod`.

        :param int height:
            The height of the :class:`Disk` to move.
        :param Rod start:
            The :class:`Rod` to move the :class:`Disk` from.
        :param Rod end:
            The :class:`Rod` to move the :class:`Disk` to.
        :param Rod tmp:
            The intermediary :class:`Rod` to use when moving the :class:`Disk`.
        """
        if height >= 1:
            for i in self.move_tower(height - 1, start, tmp, end):
                yield i
            for i in self.move_disk(start, end):
                yield i
            for i in self.move_tower(height - 1, tmp, end, start):
                yield i
        elif height == 1:
            for i in self.move_disk(start, end):
                yield i

    def move_disk(self, start, end):
        """
        Move the `Disk` from one Rod to another.

        :note:
            Generator, yields `Move` instances.
        :param Rod start:
            The :class:`Rod` to remove the :class:`Disk` from.
        :param Rod end:
            The :class:`Rods` to move the :class:`Disk` to.
        """
        start_rod = copy.deepcopy(start)
        end_rod = copy.deepcopy(end)
        moves = self.moves

        disk = start.pop()

        move = Move(disk, start_rod, end_rod, moves)

        end.append(disk)
        self._moves += 1

        yield move



#!/usr/bin/env python
# -*- coding: latin-1 -*-
#
# @module towers.core.rods
# @version 0.1
# @copyright (c) 2017-present Francis Horsman.

import copy
import json
from collections import Sequence, namedtuple

import six

from .disk import Disk
from .errors import InvalidRod, InvalidRodHeight
from .rod import Rod
from .utils import Serializable
from .validation import Validatable, validate_height

__all__ = ['Rods']


class Rods(namedtuple('Rods', ('start', 'end', 'tmp')), Sequence, Validatable, Serializable):
    """
    A collection of 3 Rod's that form the Tower.

    :param Rod start:
        The rod containing the disks at their start position.
    :param Rod end:
        The rod containing the disks at their end position.
    :param Rod tmp:
        The intermediary rod.
    :param int height:
        The height of the tower.
    :raises InvalidTowerHeight:
        The height of the tower is invalid.
    :raises InvalidRod:
        A rod is not of expected type `Rod`.
    :raises InvalidRodHeight:
        A rod height is inconsistent with the specified height.
    :raises DuplicateDisk:
        A rod contains a duplicate disk
    :raises CorruptRod:
        A disk is on top of a disk of smaller size on a Rod.
    """

    def __new__(cls, height=1, start=None, end=None, tmp=None):
        validate_height(height)

        start_rod = [Disk(rod, height) for rod in range(height)]

        for rod in [start, end, tmp]:
            if rod is None:
                continue
            if not isinstance(rod, Rod):
                raise InvalidRod(rod)
            elif rod.height != height:
                raise InvalidRodHeight(rod, height)
            rod.validate()

        if start is None:
            start = Rod(
                name='start',
                disks=start_rod,
                height=height,
            )
        if end is None:
            end = Rod(
                name='end',
                height=height,
            )
        if tmp is None:
            tmp = Rod(
                name='tmp',
                height=height,
            )

        self = super(Rods, cls).__new__(cls, start, end, tmp)
        self._height = height
        return self

    def to_json(self):
        """
        Return a json serializable representation of this instance.

        :rtype:
            object
        """
        return {
            'height': self.height,
            'start': self.start.to_json(),
            'end': self.end.to_json(),
            'tmp': self.tmp.to_json(),
        }

    @classmethod
    def from_json(cls, d):
        """
        Return a class instance from a json serializable representation.

        :param str|dict d:
            The json or decoded-json from which to create a new instance.
        :rtype:
            :class:`Rods`
        :raises:
            See :class:`Rods`.__new__.
        """
        if isinstance(d, six.string_types):
            d = json.loads(d)
        return cls(
            height=d.pop('height'),
            start=Rod.from_json(d.pop('start')),
            end=Rod.from_json(d.pop('end')),
            tmp=Rod.from_json(d.pop('tmp')),
        )

    @property
    def height(self):
        """
        Retrieve the height of the rods (ie: max number of disks each one can hold).

        :rtype:
            int
        """
        return self._height

    def __copy__(self):
        """
        Return a shallow copy of this instance.

        :rtype:
            Rods
        """
        return Rods(
            height=self.height,
            start=copy.copy(self.start),
            end=copy.copy(self.end),
            tmp=copy.copy(self.tmp),
        )

    def __deepcopy__(self, *a):
        """
        Return a deep copy of this instance.

        :rtype:
            Rods
        """
        return Rods(
            height=self.height,
            start=copy.deepcopy(self.start),
            end=copy.deepcopy(self.end),
            tmp=copy.deepcopy(self.tmp),
        )

    def __eq__(self, other):
        """
        Compare Rods instances for equivalence.

        :param Rods other:
            The other Rods to compare.
        :rtype: bool
        """
        if isinstance(other, Rods):
            if all([getattr(self, field) == getattr(other, field) for field in self._fields]):
                return True

    def __str__(self):
        return 'Rods({height} - {start}, {end}, {tmp})'.format(
            height=self.height,
            start=self.start,
            end=self.end,
            tmp=self.tmp,
        )

    def __bool__(self):
        """
        A Rods is considered True if it contains any disks on any rods.

        :rtype:
            bool
        """
        return self.__nonzero__()

    def __nonzero__(self):
        """
        A Rods is considered non-zero if it contains any disks on any rods.

        :rtype:
            bool
        """
        return any([bool(i) for i in self])

    def __len__(self):
        """
        Obtain the number of Rods.

        :rtype:
            int
        """
        return len(self._fields)

    def __iter__(self):
        """
        Iterate over all the rods.

        :rtype:
            Rod
        """
        for name in self._fields:
            yield getattr(self, name)

    def validate(self):
        """
        Perform self validation.

        :raises DuplicateDisk:
            This rod already contains this disk
        :raises CorruptRod:
            A disk is on top of a disk of smaller size.
        """
        for rod in self:
            rod.validate()



# !/usr/bin/env python
# -*- coding: latin-1 -*-
#
# @module towers.core.moves
# @version 0.1
# @copyright (c) 2017-present Francis Horsman.

from collections import namedtuple

__all__ = [
    'Move',
]


class Move(namedtuple('Move', ('disk', 'start', 'end', 'moves'))):
    """
    :param Disk disk:
        The disk that will be moved.
    :param Rod start:
        The state of the start_rod prior to the move.
    :param Rod end:
        The state of the end_rod prior to the move.
    :param int moves:
        The number of moves prior to the move.
    """

    def __new__(cls, disk, start, end, moves):
        return super(Move, cls).__new__(cls, disk, start, end, moves)



#!/usr/bin/env python
# -*- coding: latin-1 -*-
#
# @module towers.core.errors
# @version 0.1
# @copyright (c) 2017-present Francis Horsman.

__all__ = [
    'TowersError',
    'DuplicateDisk',
    'CorruptRod',
    'InvalidStartingConditions',
    'InvalidEndingConditions',
    'InvalidTowerHeight',
    'InvalidDiskPosition',
    'InvalidRod',
    'InvalidRodHeight',
    'InvalidRods',
    'InvalidMoves',
]


class TowersError(Exception):
    """
    Base class of all `towers` errors.
    """
    pass


class InvalidRod(TypeError, TowersError):
    def __init__(self, rod):
        """

        :param object rod:
            The :class:`Rod` which is invalid.
        """
        super(InvalidRod, self).__init__(
            'invalid rod: {rod}'.format(rod=rod))
        self.rod = rod


class InvalidRods(TypeError, TowersError):
    def __init__(self, rods):
        """
        :param object rods:
            The :class:`Rods` which are invalid
        """
        super(InvalidRods, self).__init__(
            'invalid rod: {rods}'.format(
                rods=rods))
        self.rods = rods


class InvalidRodHeight(ValueError, TowersError):
    def __init__(self, rod, max_height):
        """
        :param Rod rod:
            The :class:`Rod` which has an invalid height.
        :param int max_height:
            The max allowed height of the :class:`Rod`.
        """
        super(InvalidRodHeight, self).__init__(
            'invalid rod height: {rod} expecting: {height}.'.format(
                rod=rod, height=max_height))
        self.rod = rod
        self.height = max_height


class DuplicateDisk(ValueError, TowersError):
    """
    A duplicate disk was found on a tower.
    """

    def __init__(self, rod, disk_width):
        """
        :param Rod rod:
            The duplicate :class:`Rod`.
        :param int disk_width:
            The width of the :class:`Disk`.
        """
        super(DuplicateDisk, self).__init__(
            'Duplicate disk width found: {disk_width} in: {rod}'.format(
                disk_width=disk_width, rod=rod))
        self.rod = rod
        self.disk_width = disk_width


class CorruptRod(ValueError, TowersError):
    """
    A :class:`Rod` with an invalid stack of disks was found.
    """

    def __init__(self, rod, disk):
        """
        :param Rod rod:
            The :class:`Rod` which is corrupt.
        :param int disk:
            A :class:`Disk` which sits directly atop a smaller :class:`Disk`.
        """
        super(CorruptRod, self).__init__(
            'Corrupt rod, at least one disk is larger than the one below it: {rod}'.format(
                rod=rod))
        self.rod = rod
        self.disk = disk


class InvalidStartingConditions(ValueError, TowersError):
    """
    The :class:`Rods` for the towers are not in the correct starting state.
    """

    def __init__(self, rods, moves):
        """
        :param Rod rods:
            The :class:`Rod`'s.
        :param int moves:
            Total number of moves already made (should be zero).
        """
        super(InvalidStartingConditions, self).__init__(
            'Invalid starting condition for rods: {rods}, with existing moves: {moves}'.format(
                rods=rods, moves=moves))
        self.rods = rods
        self.moves = moves


class InvalidEndingConditions(ValueError, TowersError):
    """
    The :class:`Rod`'s for the towers are not in the correct ending state.
    """

    def __init__(self, rods):
        """
        :param Rod rods:
            The :class:`Rod`'s.
        """
        super(InvalidEndingConditions, self).__init__(
            'Invalid ending condition for rods: {rods}'.format(
                rods=rods))
        self.rods = rods


class InvalidTowerHeight(ValueError, TowersError):
    """
    The height of the :class:`Tower` is invalid.
    """

    def __init__(self, height):
        """
        :param int height:
            The invalid height.
        """
        super(InvalidTowerHeight, self).__init__(
            'Invalid tower height: {height}'.format(
                height=height))
        self.height = height


class InvalidDiskPosition(ValueError, TowersError):
    """
    The position of the :class:`Disk` is invalid.
    """

    def __init__(self, position, height):
        """
        :param int position:
            The invalid position on the :class:`Rod`.
        :param int height:
            The height.
        """
        super(InvalidDiskPosition, self).__init__(
            'Invalid disk position: {position} on Rod of height: {height}'.format(
                position=position, height=height))
        self.position = position
        self.height = height


class InvalidMoves(ValueError, TowersError):
    """
    An invalid number of moves.
    """

    def __init__(self, moves):
        """
        :param int moves:
            The invalid `moves`.
        """
        super(InvalidMoves, self).__init__(
            'Invalid moves: {moves}'.format(
                moves=moves))
        self.moves = moves



#!/usr/bin/env python
# -*- coding: latin-1 -*-
#
# @module towers.core.disk
# @version 0.1
# @copyright (c) 2017-present Francis Horsman.

import json
from collections import namedtuple

import six

from .errors import InvalidDiskPosition
from .utils import Serializable
from .validation import Validatable, validate_height

__all__ = ['Disk']


class Disk(namedtuple('Disk', ('original_position', 'height')), Validatable, Serializable):
    """
    An immutable representation of a sized disk that sits on a `Rod`.
    """

    def __new__(cls, original_position, height=1):
        """
        :param int original_position:
            The position on the `Rod` that this disks originally sat.
            Zero = The bottom of the `Rod`.
        :param int height:
            The maximum position of this :class:`Disk` on a :class:`Rod`.
        :rtype:
            :class:`Disk`
        :raises InvalidTowerHeight:
            The height of the tower is invalid.
        :raises InvalidDiskPosition:
            The position of the disk is invalid.
        """
        self = super(Disk, cls).__new__(
            cls,
            original_position,
            height,
        )
        self.validate()
        return self

    def to_json(self):
        """
        Return a json serializable representation of this instance.

        :rtype:
            object
        """
        return {
            'original_position': self.original_position,
            'height': self.height,
        }

    @classmethod
    def from_json(cls, d):
        """
        Return a class instance from a json serializable representation.

        :param str|dict d:
            The json or decoded-json from which to create a new instance.
        :rtype:
            :class:`Disk`
        :raises:
            See :class:`Disk.__new__`.
        """
        if isinstance(d, six.string_types):
            d = json.loads(d)
        return cls(
            original_position=d.pop('original_position'),
            height=d.pop('height'),
        )

    @property
    def width(self):
        """
        Obtain the width of the disk

        :rtype: int
        """
        return self.height - self.original_position

    def validate(self):
        """
        Perform self validation

        :raises InvalidTowerHeight:
            The height of the tower is invalid.
        :raises InvalidDiskPosition:
            The position of the disk is invalid.
        """
        validate_height(self.height)
        if (self.original_position < 0) or (self.original_position >= self.height):
            raise InvalidDiskPosition(self.original_position, self.height)

    def __str__(self):
        return 'Disk({original_position})'.format(
            original_position=self.original_position)

    def __repr__(self):
        return '{original_position}'.format(
            original_position='*' * self.width)



#!/usr/bin/env python
# -*- coding: latin-1 -*-
#   ______
#  /\__  _\
#  \/_/\ \/   ___   __  __  __    __  _ __  ____
#     \ \ \  / __`\/\ \/\ \/\ \ /'__`/\`'__/',__\
#      \ \ \/\ \L\ \ \ \_/ \_/ /\  __\ \ \/\__, `\
#       \ \_\ \____/\ \___x___/\ \____\ \_\/\____/
#        \/_/\/___/  \/__//__/  \/____/\/_/\/___/
#
#

import os
import sys

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages

if sys.version_info[0] == 2:
    if sys.version_info[1] < 6:
        raise ValueError('Only compatible with python; >= 2.6')

here = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(here, 'towers', '__version__.py')) as f:
    exec (f.read(), about)

# 'setup.py publish' shortcut.
if sys.argv[-1] == 'publish':
    version = about['__version__']
    print("Tagging release as: {v}".format(v=version))
    os.system("git tag -a {v} -m 'version {v}'".format(v=version))
    os.system('git push --tags')
    os.system('python setup.py sdist bdist_wheel')
    os.system('twine upload dist/*')
    sys.exit()
elif sys.argv[-1] == 'test':
    os.system('make test')
    sys.exit()

requires = [
    'six',
    'pip',
    'enum34',
]

setup_requirements = [
    'nose',
]
test_requirements = [
    'nose',
    'nose-cov',
]

setup(
    name=about['__title__'],
    version=about['__version__'],
    description=about['__description__'],
    author=about['__author__'],
    author_email=about['__author_email__'],
    url=about['__url__'],
    packages=find_packages(include=['towers']),
    # package_data={
    #     'requirements': ['requirements/*.txt'],
    # },
    include_package_data=True,
    license=about['__license__'],
    requires=requires,
    install_requires=requires,
    zip_safe=False,
    keywords='towers',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    test_suite='tests',
    tests_require=test_requirements,
    setup_requires=setup_requirements,
)



#!/usr/bin/env python
# -*- coding: latin-1 -*-
#
# @module towers
# @version 0.1
# @copyright (c) 2017-present Francis Horsman.
#
#   ______ _____   __      __  ____    ____    ____
#  /\__  _/\  __`\/\ \  __/\ \/\  _`\ /\  _`\ /\  _`\
#  \/_/\ \\ \ \/\ \ \ \/\ \ \ \ \ \L\_\ \ \L\ \ \,\L\_\
#     \ \ \\ \ \ \ \ \ \ \ \ \ \ \  _\L\ \ ,  /\/_\__ \
#      \ \ \\ \ \_\ \ \ \_/ \_\ \ \ \L\ \ \ \\ \ /\ \L\ \
#       \ \_\\ \_____\ `\___x___/\ \____/\ \_\ \_\ `\____\
#        \/_/ \/_____/'\/__//__/  \/___/  \/_/\/ /\/_____/
#
#

from .core.disk import Disk
from .core.errors import (
    CorruptRod, DuplicateDisk, InvalidDiskPosition, InvalidEndingConditions, InvalidMoves,
    InvalidRod, InvalidRodHeight, InvalidRods, InvalidStartingConditions, InvalidTowerHeight,
    TowersError,
)
from .core.moves import Move
from .core.rod import Rod
from .core.rods import Rods
from .core.towers import Towers
from .core.validation import validate_height, validate_moves, validate_rods
from .__version__ import __version__, __author__, __title__

__all__ = [
    'Towers',
    'Disk',
    'Rod',
    'Rods',
    'Move',
    'TowersError',
    'DuplicateDisk',
    'CorruptRod',
    'InvalidStartingConditions',
    'InvalidEndingConditions',
    'InvalidTowerHeight',
    'InvalidDiskPosition',
    'InvalidRod',
    'InvalidRods',
    'InvalidRodHeight',
    'InvalidMoves',
    'validate_height',
    'validate_rods',
    'validate_moves',
]
